@@ -638,7 +638,7 @@ sub error {
 	my $class=shift;
 	print STDERR "E: ".shift()."\n";
 	# Error unwind.
-	foreach (reverse @unwind) {
+	while ($_ = pop @unwind) {
 		$_->();
 	}
 	exit(1);
@@ -657,14 +657,32 @@ sub info {
 # Runs a shell command, gaining root if necessary.
 sub do_root {
 	my $class=shift;
-	if ($> != 0) {
-		my @command=qw(sudo);
+	my $interpolated = 0;
+	my @command;
+	if ($> == 0) {
+		@command = @_;
+	} else {
 		if ($_config->exists('APT::Src::RootCommand')) {
 			@command=split(/\s+/, $_config->get('APT::Src::RootCommand'));
+		} else {
+			@command = qw(sudo);
+		}
+		# look for %s in the command to see if the user requests
+		# interpolation instead of just appending
+		foreach (@command) {
+			if (/%s/) {
+				$interpolated = 1;
+				$_ = sprintf($_, join(' ', @_));
+				last;
+			}
+		}
+		# probably sudo, which doesn't require the command to be passed
+		# as a single string, so append the array
+		if (!$interpolated) {
+			push @command, @_;
 		}
-		unshift @_, @command;
 	}
-	$class->do(@_);
+	$class->do(@command);
 }
 
 # Runs a shell command, only displaying its output in verbose mode or if it
